\name{CopulaEnv}

\alias{CopulaEnv}


\title{Bivariate Copula Environment}


\description{
    
    Set and Get functions for the Copula environment.
    
}


%\usage{}


%\arguments{}


%\value{}


%\examples{}



\keyword{models}

